/**
 * CompInfoObject Class:
 * Properties:
 *  object_type:'CIO' - Compononent Info type object
 *  parCActivityDIV_element:null - contains the the parentNode with title = parentCActivity_title
 *  buiDIV_element:null - contains the reference node with class = builder_class
 *  instructions_array:new Array() -  an array that has the instructions on how to build this component
 *  activity_ID:null - contains a unique id which can be used later if needed
 * 
 * Constructor:
 *  constructor(ref_parCActivityDIV,builder_class_name) - creates an object and sets parCActivityDIV_element and buiDIV_element references based on arguements
 * 
 * Methods:
 *  pop_instruction() - FIFO get instructions
 *  getIA_Size() - get the number of instructions in the array
 * 
 */

class CompInfoObject{
    object_type='CIO';
    parCActivityDIV_element=null;
    buiDIV_element=null;
    instructions_array=new Array();
    activity_ID=null;

    constructor(ref_parCActivityDIV,builder_class_name){
        if(ref_parCActivityDIV && builder_class_name){
            this.parCActivityDIV_element = ref_parCActivityDIV;
            const ref_builder_class = SearchChildrenFor([['tagName','DIV'],['class',builder_class_name]],ref_parCActivityDIV);
            if(ref_builder_class){
                this.buiDIV_element = ref_builder_class;
            }
        }
    }

    pop_instruction(){
        return this.instructions_array.shift();
    }
    getIA_Size(){
        return this.instructions_array.length;
    }
    toString(){
        let tstr = "PRINTING contents of "+this.object_type;
        tstr += "\nparent activity reference tagName: ["+this.parCActivityDIV_element.tagName+"]";
        tstr += "\nparent activity reference class: ["+this.parCActivityDIV_element.className+"]";
        tstr += "\nbuilder reference tagName: ["+this.buiDIV_element.tagName+"]";
        tstr += "\nbuilder reference class: ["+this.buiDIV_element.className+"]";
        tstr += "\nActivity ID: ["+this.activity_ID+"]";
        tstr += "\nInstructions array size: "+this.getIA_Size();
        return tstr;
    }
}

/**
 * InstructObject Class:
 * Properties:
 *  object_type:'INO' - Instruction type object
 *  instruction:null - The instruction array, ['instruction','args'...,'args']
 *  row_info_arrays:new Array(); - contains an array of individual row by info_col arrays used for the instructions
 *  areObjects_RIA:false - default row_info_arrays (RIA) is an array or row arrays
 *  randomize_ME:false - default is false, not to randomize the component
 *  rnd_order_arr:null - holds the random order to build the component
 *  settings_arr:new Map() - holds any settings found in the builder ([data-setting-type,value]...,[data-setting-type,value])
 * 
 * Constructor:
 *  constructor(instruct) - creates an object and sets instructions to the instructions array passed in
 * 
 * Methods:
 *  get_row_At(indx){} - get the row array at specified index
 *  getIA_Size() - get the number of Info arrays contained, ie. number of TRs
 *  getRow_IA_Size() - get the number of columns contained in one row of the info arrays
 *  get_info_At_ColNUM(indx,colNum) - get info at column using the colum number int
 *  set_RND_order() - sets the random order to build the component
 *  get_SAR_value(attr) - get the value for the settings attribute
 * 
 */

class InstructObject {
    object_type='INO';
    instruction=null;
    row_info_arrays=new Array();
    areObjects_RIA=false;
    randomize_ME=false;
    rnd_order_arr=null;
    settings_level=null;
    settings_arr=new Map();
    
    constructor(instruct){
        this.instruction = instruct;
    }

    get_row_At(indx){
        return this.row_info_arrays[indx];
    }
    getIA_Size(){
        return this.row_info_arrays.length;
    }
    getRow_IA_Size(){
        return this.row_info_arrays[0].length;
    }
    get_info_At_ColNUM(indx,colNum){
        let row_info_array = this.get_row_At(indx);
        return row_info_array[colNum];
    }
    set_info_At_ColNUM(indx,colNum,val){
        this.row_info_arrays[indx][colNum] = val;
    }
    set_RND_order(){
        let count = this.getIA_Size();
        let i;
        let rnd_order = new Array();
        for(i=0; i<count; i++){
            let isRandNumFound = false;
            while(!isRandNumFound){
                let rnd_num = Math.floor(Math.random() * count);
                if(rnd_order.length==0 || !rnd_order.includes(rnd_num)){
                    rnd_order.push(rnd_num);
                    isRandNumFound = true;
                }
            }
        }
        this.rnd_order_arr = rnd_order;
        this.randomize_ME = true;
    }
    get_SAR_value(attr){
        return this.settings_arr.get(attr);
    }
    toString(){
        let tstr = "PRINTING contents of "+this.object_type;
        tstr += "\nRow_Info_Array for Instructions count:"+this.getIA_Size();
        tstr += "\nInstructions Array[ "+this.instruction+" ]";
        tstr += "\nRandomizeMe Value [ "+this.randomize_ME+" ]";
        if(this.rnd_order_arr){
            tstr += "\n Randomize array toString: ["+this.rnd_order_arr.toString()+"]";
        }
        if(this.settings_arr){
            tstr += "\nSettings Map toString: {"
            for(const [key,value] of this.settings_arr){
                tstr += "["+key+" -> "+value+"]";
            }
            tstr += "}";
        }
        if(this.settings_level){
            tstr += "\nSettings level: ["+this.settings_level+"]";
        }
        tstr += "\nrow_info_array toString";
        let row;
        for(row=0;row<this.getIA_Size();row++){
            let col;
            for(col=0;col<this.getRow_IA_Size();col++){
                tstr += "\n["+this.get_info_At_ColNUM(row,get_col_int("[-COL_"+col+"-]")).tagName+"] Content: ["+this.get_info_At_ColNUM(row,get_col_int("[-COL_"+col+"-]")).innerHTML+"], with class: ["+this.get_info_At_ColNUM(row,get_col_int("[-COL_"+col+"-]")).className+"]";
            } 
        }
        return tstr;
    }
}

let STRUCT_Stack; //(STS)
let OPEN_ELEMENT_Stack; //Store the index of where the last open element was when a new element is created (OES)
let sParts;
let dParts;
let setting_ele_type='DIV';
let setting_class='component-settings'; //some components might have setting attached to it
let setting_span_attr='data-setting-type'; //the data attribute to identify the li which list the settings


function Find_Build_Settings_Array(ref_buiDIV_ele,ref_instruct_obj){
    //check to see if it has the tagName OL and class "settings-list"
    if(ref_instruct_obj.settings_level){
        const settings_level_found = SearchChildrenFor([['tagName',"SECTION"],['class',ref_instruct_obj.settings_level]],ref_buiDIV_ele);
        if(settings_level_found){
            const ol_ele_found = SearchChildrenFor([['tagName',setting_ele_type],['class',setting_class]],settings_level_found);
            if(ol_ele_found){
                //for each li children get the data-setting-type attribute and it's value
                //need to check to see if the settings are in an ol list with class="settings-list" or just just p tags
                let get_settings_from_ele = ol_ele_found;
                const search_for_ol = SearchChildrenFor([['tagName','OL'],['class','settings-list']],ol_ele_found);
                if(search_for_ol){
                    get_settings_from_ele = search_for_ol;
                }
                const ol_ele_found_childs = get_settings_from_ele.children;
                if(ol_ele_found_childs.length>0){
                    let i;
                    for(i=0;i<ol_ele_found_childs.length;i++){
                        const search_for_span = SearchChildrenFor([['tagName','SPAN'],[setting_span_attr,'*']],ol_ele_found_childs[i]);
                        if(search_for_span){
                            ref_instruct_obj.settings_arr.set(search_for_span.getAttribute(setting_span_attr),search_for_span.textContent.trim());
                        }
                    }
                }
            }
        }
    }
}

function Grab_Content(ref_element, content_type, content_div){
    const compContentChild = ref_element.children;
    if(content_type=="TEXT"){
        let compName;
        if(compContentChild.length>0){
            //text might be encapsulated in other tags
            compName = Get_TEXT(compContentChild);
        }else{
            //check to see if it's just textContent without any children
            compName = ref_element.textContent;
        }
        if(compName && compName.trim().length > 0){
            let textNodeFound = document.createTextNode(compName);
            content_div.append(textNodeFound);
            return true;
        }
    }else if (content_type=="MIXED"){
        if(compContentChild.length>0){
            content_div.innerHTML = ref_element.innerHTML;
            return true;
        }else{
            const justText = ref_element.textContent;
            if (justText && justText.trim() !=""){
                const textNodeFound = document.createTextNode(justText);
                content_div.append(textNodeFound);
                return true;
            }
        }
    }
    return false;
}

function get_col_int(colStr){ //get info at column using the colum string format [-COL_#-] usually from the STRUCT
    //parse colStr example [-COL_0-]
    let rep01 = colStr.replace(/"/g, "");
    let rep02_1 = rep01.replace(/\[-COL_/g, "");
    let rep02_2 = rep02_1.replace(/-]/g, "");
    let rep03 = rep02_2.trim();
    let col = parseInt(rep03);
    return col;
}
function get_SD_index_int(str){
    let rep01 = str.replace(/D_/g, "");
    let rep02 = rep01.replace(/S_/g, "");
    let rep03 = rep02.replace(/R_/g, "");
    let indx = parseInt(rep03.trim());
    return indx;
}
function get_SAR_attr(str){
    let rep01 = str.replace(/"/g, "");
    let rep02_1 = rep01.replace(/\[SAR_/g, "");
    let rep02_2 = rep02_1.replace(/]/g, "");
    let rep03 = rep02_2.trim();
    return rep03;
}

function Build_HTML(ref_obj){
    if(ref_obj.getIA_Size()>0){
        STRUCT_Stack = new Array();
        OPEN_ELEMENT_Stack = new Array(); //push in when new element tagName is created, pop when closing / pair is found
        let ins_count;
        for(ins_count=0;ins_count<ref_obj.getIA_Size();ins_count++){
            let instruct_obj = ref_obj.pop_instruction();
            STRUCT_Parser(instruct_obj);
        }
        let htmlCodeHolder=STRUCT_Stack.pop();
        return htmlCodeHolder;
    }
}

function insert_before(ref_parentDiv,ref_insert_before_ele, ref_html_code){
    let htmlCodeHolder_Children = ref_html_code.children;
    if(htmlCodeHolder_Children.length>0){
        while(htmlCodeHolder_Children.length>0){
            ref_parentDiv.insertBefore(htmlCodeHolder_Children[0],ref_insert_before_ele);
        }
    }
    
    ref_insert_before_ele.remove();
}

function insert_after(ref_insert_after_ele, ref_html_code){
    let htmlCodeHolder_Children = ref_html_code.children;
    if(htmlCodeHolder_Children.length>0){
        while(htmlCodeHolder_Children.length>0){
            const child_to_insert = htmlCodeHolder_Children[(htmlCodeHolder_Children.length)-1];
            ref_insert_after_ele.after(child_to_insert);
        }
    }
    
    ref_insert_after_ele.remove();
}

function insert_into(ref_insert_into_ele, ref_html_code){
    let htmlCodeHolder_Children = ref_html_code.children;
    if(htmlCodeHolder_Children.length>0){
        while(htmlCodeHolder_Children.length>0){
            const child_to_insert = htmlCodeHolder_Children[0];
            ref_insert_into_ele.append(child_to_insert);
        }
    }
}

function Check_Apply_Settings(inf_arr_obj){
    let s;
    for(const [key,value] of inf_arr_obj.settings_arr){
        if(key=='randomize-me' && (value.toLowerCase()=='y' || value.toLowerCase()=='yes')){
            inf_arr_obj.set_RND_order();
        }
    }
}

/**
 * Understands the STRUCT structure, and knows how to deal with different instruction commands
 * @param {Object or Array} raw_ingredients instruction object or an array in the format [instructions, row #, row of Info columns]
 * 
 * Instructions Commands:
 *      -> ELEMENT, '','',... : element to create using the 'document.createElement("ELEMENT")' and then add in the classes and attributes passed in as arguement(s)
 *      -> CREATE_TEXT_NODE, 'STRING': passed in STRING calls on the 'document.createTextNode(STRING)'
 *      -> CREATE_TEXT_NODE, '[-COL_#-]': passed in column string format, get the info from the info array at that column number (might be string or a div with a textNode child) and then calls on the 'document.createTextNode(string from COL_#)' 
 *      -> GET_CONTENT, '[-COL_#-]': passed in column string format, get the html elements from the info array at that column number and pop into array
 *      -> BUILD_INFO_ARRS,'R_#' (optional): passed in row index of the instructions object found in the row_info_array to pass to the struct parser to process, or 
 *      -> GET_INSTRUCTION, '[-COL_#-]': passed in column string format, get the STRUCT instruction info array at that column number. Use when want to direct builder based on user input or configuration
 *      -> : 
 */

function STRUCT_Parser(raw_ingredients){
    let raw_ingredients_type = 0; //1 - array, 2 - object, 0 - not those
    let row_arr;
    let row_index;
    let arr_instructionS;
    if(Object.toString(raw_ingredients).constructor.toString().indexOf("Object") > -1 || raw_ingredients.constructor.toString().indexOf("class") > -1){
        raw_ingredients_type = 2;
        if(raw_ingredients.areObjects_RIA){ //run instructions and pass in objects when need to loop/build info arrays full of instruction objects
            arr_instructionS = raw_ingredients.instruction;
        }else{
            //if row_info_array is full of arrays then must be row specific info to build?, pass in instructions and then the row info?   
            let row;
            for(row=0; row<raw_ingredients.getIA_Size(); row++){
                let info_row_to_get = row;
                if(raw_ingredients.randomize_ME){
                    info_row_to_get = raw_ingredients.rnd_order_arr[row];
                }
                let r_ingred = new Array();
                r_ingred.push(raw_ingredients.instruction); //instructions
                r_ingred.push(row); //leave as row since we need natural component number to be natural
                r_ingred.push(raw_ingredients.get_row_At(info_row_to_get));
                STRUCT_Parser(r_ingred);
            }
        }
    }else if(raw_ingredients.constructor.toString().indexOf("Array") > -1){
        raw_ingredients_type = 1;
        arr_instructionS = raw_ingredients[0];
        row_index = raw_ingredients[1]; 
        row_arr = raw_ingredients[2];
    }
    if(arr_instructionS){
        let i;
        for(i=0;i<arr_instructionS.length;i++){
            let j;
            let arr_instruction = arr_instructionS[i];
            let ins_part;
            let ins_type; //type of instruction: get content (1), pop instruction from array (2), create text node (3), create element (4)
            let args_count=0;
            let ref_ele;
            for(j=0;j<arr_instruction.length;j++){
                if(j==0){ //instruction iPart, call
                    ins_part = arr_instruction[j];
                    if(ins_part){
                        ins_part = ins_part.trim();
                    }
                    let ins_args = arr_instruction[j+1];
                    if(ins_args){
                        ins_args = ins_args.trim();
                    }
                    
                    if(ins_part == "GET_CONTENT"){ //get content (1)
                        ins_type = 1;
                        //get the content at column indicated by [-COL_#-] and push into the STRUCT_Stack and then break
                        if(raw_ingredients_type == 1){
                            let content_to_push;
                            let content_found_to_push = row_arr[get_col_int(ins_args.trim())];
                            const content_found_to_push_childs = content_found_to_push.childNodes;
                            if(content_found_to_push_childs.length>0){
                                for(const content_to_push of content_found_to_push_childs){
                                    STRUCT_Stack.push(content_to_push);
                                }
                            }
                        }
                        break;
                    }else if(ins_part == "BUILD_INFO_ARRS"){ //loop in dynamic parts (2)
                        ins_type = 2;
                        if(raw_ingredients_type == 2){ //an object
                            //call STRUCT_Parser with the dynamic arr instruction at index passed in then break
                        
                            //for each check to see if it's an object or an array
                            //check to make sure areObjects_RIA is True
                            if(raw_ingredients.areObjects_RIA){ //if row_info_array are full of objects, it's an instruction object then just call STRUCT_Parser and pass in the instruction object
                                if(ins_args){
                                    const sd_index = get_SD_index_int(ins_args.trim());
                                    //check to make sure there's an index specified SD
                                    if(sd_index<raw_ingredients.row_info_arrays.length){
                                        STRUCT_Parser(raw_ingredients.row_info_arrays[sd_index]);
                                    }
                                }else{
                                    let ins_row;
                                    for(ins_row=0; ins_row<raw_ingredients.getIA_Size(); ins_row++){
                                        let info_row_to_get = ins_row;
                                        if(raw_ingredients.randomize_ME){
                                            info_row_to_get = raw_ingredients.rnd_order_arr[ins_row];
                                        }
                                        STRUCT_Parser(raw_ingredients.row_info_arrays[info_row_to_get]);
                                    }
                                }
                            }
                        }else if(raw_ingredients_type == 1){ //an array
                            //passed specific row info array with content and object inside to process
                            if(ins_args){
                                STRUCT_Parser(row_arr[get_col_int(ins_args.trim())]);
                            }
                        }
                        break;
                    }else{
                        if(ins_part == "CREATE_TEXT_NODE"){ //create text node (3) , used when only want the text inside the div
                            ins_type = 3;
                            let txtNode;
                            let struct_content;
                            //check to see if the found COL, SAR, ROW in the 0 index of the val string. if not then need to do a replace 
                            if(ins_args.trim().indexOf("[-COL_") > -1 || ins_args.trim().indexOf("[SAR_") > -1 || ins_args.trim().indexOf("[-ROW_Num") > -1){
                                let val = ins_args;
                                const extract_val_arr = Extract_ValCode(val.trim());
                                if(extract_val_arr){
                                    let val_counter;
                                    for(val_counter=0;val_counter<extract_val_arr.length;val_counter++){
                                        const arg_attr = extract_val_arr[val_counter];
                                        if(arg_attr == "[-ROW_Num-]"){
                                            //ROW_Num starting at 1, add 1 to array index
                                            val = val.replace(arg_attr, row_index+1);
                                        }else if(arg_attr.indexOf("[SAR_") > -1){ //need to get content from settings arr with attribute passed in
                                            //need to extract the [SAR_] portion
                                            //need to get the value based on attribute
                                            const sar_attr = get_SAR_attr(arg_attr);
                                            const sar_content = raw_ingredients.get_SAR_value(sar_attr);
                                            if(sar_content){
                                                val = val.replace(arg_attr, sar_content);
                                            }else{
                                                val = val.replace(arg_attr, '');
                                            }
                                        }else if(arg_attr.indexOf("[-COL_") > -1){
                                            const col_content = row_arr[get_col_int(arg_attr)];
                                            val = val.replace(arg_attr, col_content.textContent);
                                        }
                                        val= val.replace(/"/g, "");
                                        val= val.replace(/&nbsp;/g, "");
                                    }
                                    struct_content = val;
                                }
                            }else{
                                //just static text found in the STRUCT
                                struct_content = ins_args.trim();
                            }
                            if(struct_content.constructor.toString().indexOf("String") > -1){
                                txtNode = document.createTextNode(struct_content);
                            }else{
                                if(struct_content.tagName == "DIV"){
                                    const attributes = struct_content.getAttributeNames();
                                    if(attributes.length == 0){
                                        if(struct_content && struct_content.textContent){
                                            //empty div just take the innerNode
                                            txtNode = document.createTextNode(struct_content.textContent);
                                        }else{
                                        }
                                    }else{
                                        //there are classes and attributes in there
                                        txtNode = struct_content;
                                    }
                                }else{
                                    txtNode = struct_content;
                                }
                                
                            }
                            if(txtNode){
                                STRUCT_Stack.push(txtNode);
                                if(txtNode.textContent){
                                }else{
                                }
                            }else{
                            }
                            break;
                        }else if(ins_part == "/"){
                            //pop the last element in the stack, pop the 2nd last element, append last to 2nd last and pop back into stack
                            //when hit /, pop each element and string together as children until hit a popped element that empty inside then append stop
                            let end_tag=ins_args;
                            if(STRUCT_Stack.length>1){
                                let divHolder = document.createElement("DIV");
                                let LE;
                                let OES_last_open_index=OPEN_ELEMENT_Stack.pop(); //last open element can be found in index of the STRUCT_Stack
                                if(OES_last_open_index >= 0){ //only do things if there is a last index, if not then no elements are have been created or it's been popped out already
                                    let found_is_empty_remove_last_OE = false;
                                    while(STRUCT_Stack.length > 0){
                                        //pop LE and check to see if end of children and found empty element thats needs stuff inside
                                        let STS_lastIndex =  STRUCT_Stack.length - 1;
                                        LE = STRUCT_Stack.pop();
                                        if(STS_lastIndex == OES_last_open_index && LE.tagName == end_tag){
                                            //this needs the stuff inside
                                            if (divHolder.innerHTML || divHolder.textContent){
                                                LE.insertAdjacentHTML("beforeend",divHolder.innerHTML);
                                            }
                                            break;
                                        }else{
                                            if(LE.className == "is-empty-remove-last-OE"){
                                                //found a div holding the content that is empty and is tag with the is-empty-remove-last-OE class
                                                //will not add the direct parent of this to the stack
                                                found_is_empty_remove_last_OE = true;
                                            }
                                            //just add it to the holder
                                            if(!LE.outerHTML){
                                                //see if it's textNode and if it is use textContent
                                                if(LE.textContent){
                                                    divHolder.insertAdjacentHTML("afterbegin",LE.textContent);
                                                }
                                            }else{
                                                divHolder.insertAdjacentHTML("afterbegin",LE.outerHTML);
                                            }
                                        }
                                    }
                                    if (!found_is_empty_remove_last_OE){
                                        STRUCT_Stack.push(LE);
                                    }else{
                                        //don't push it in and then just reset the flag
                                        found_is_empty_remove_last_OE = false;
                                    }
                                }
                            }else{
                            }
                            break;
                        }else if(ins_part == "CREATE_CMT_NODE"){ //create comment node
                            ins_type = 5;
                            let cmtNode;
                            //just static text found in the STRUCT
                            let struct_content = ins_args.trim();
                            cmtNode = document.createComment(struct_content);
                            if(cmtNode){
                                STRUCT_Stack.push(cmtNode);
                            }else{
                            }
                            break;
                        }else{ //create element (4)
                            ins_type = 4;
                            ref_ele = document.createElement(ins_part);
                        }
                    }
                }else if(j>0){ //passed in arguements parts
                    if(ins_type == 4){ //args are to build the class and attributes list add to the element created (4)
                        let attribute = arr_instruction[j].trim(); //attribute or class
                        const aORc_value = attribute.split("=");
                        let aORc = aORc_value[0].trim();
                        let val=aORc_value[1].trim();
                        const extract_val_arr = Extract_ValCode(val.trim());
                        if(extract_val_arr){
                            let val_counter;
                            for(val_counter=0;val_counter<extract_val_arr.length;val_counter++){
                                const arg_attr = extract_val_arr[val_counter];
                                if(arg_attr == "[-ROW_Num-]"){
                                    //ROW_Num starting at 1, add 1 to array index
                                    val = val.replace(arg_attr, row_index+1);
                                }else if(arg_attr.indexOf("[SAR_") > -1){ //need to get content from settings arr with attribute passed in
                                    //need to extract the [SAR_] portion
                                    //need to get the value based on attribute
                                    const sar_attr = get_SAR_attr(arg_attr);
                                    const sar_content = raw_ingredients.get_SAR_value(sar_attr);
                                    if(sar_content){
                                        val = val.replace(arg_attr, sar_content);
                                    }else{
                                        val = val.replace(arg_attr, '');
                                    }
                                }else if(arg_attr.indexOf("[-COL_") > -1){
                                    const col_content = row_arr[get_col_int(arg_attr)];
                                    val = val.replace(arg_attr, col_content.textContent);
                                }
                                val= val.replace(/"/g, "");
                            }
                        }
                        
                        let val_clean = val.replace(/"/g, "");
                        if(aORc == "id"){
                            ref_ele.id=val_clean;
                        }else{
                            ref_ele.setAttribute(aORc,val_clean);
                        }
                    }
                    args_count++;
                }
            }
            if (ins_type == 4){
                let STS_Size = STRUCT_Stack.push(ref_ele);
                let indx_pushed_into = STS_Size-1;
                OPEN_ELEMENT_Stack.push(indx_pushed_into);
            }
        }
    }
}

function Extract_ValCode(val_str){
    //while can't find any more [] pairing
    const val_arr = new Array();
    while(val_str.indexOf("[") > -1 && val_str.indexOf("]") > -1){
        const startAt =val_str.indexOf("[");
        const stopAt= val_str.indexOf("]");
        if(startAt > -1 && stopAt > -1 && stopAt > startAt){
            const extract_val = val_str.substring(startAt, stopAt+1);
            val_arr.push(extract_val);
            //take out the value and check again in loop
            val_str = val_str.replace(extract_val,'');
        }
    }
    if(val_arr.length>0){
        return val_arr;
    }
    return false;
}

/**
 * Get the front and back headers
 * @param {*} compNameChildren 
 * @returns 
 */
function Get_TEXT(compNameChildren){
    let i;
    let compNameStr="";
    for (i=0; i<compNameChildren.length; i++){
        if(i==0){
            compNameStr = compNameStr + "" + (compNameChildren[i].textContent).trim();
        }else{
            compNameStr = (compNameStr).trim() + " " + (compNameChildren[i].textContent).trim();
        }
    }
    return compNameStr;
}

function Get_UNQ_RNDid(){
    const uName = '' + Math.random().toString(36).substring(2,9); //default unique name is random gen
    return uName;
}